(function ($) {
    var defaultOptions = {
        'convertMail': true,
        'convertPhone': false,
        'convertURL': true,
        'urlTarget': '_blank',
        'urlSchemas': ['http', 'https']
        //'mailFilter': function (aElement) { },
        //'phoneFilter': function (aElement) { },
        //'urlFilter': function (aElement) { }
    };

    ///
    /// クラスメソッド
    ///
    $.detectHref = {
        urlPatternAfterSchema: ':\\/\\/[\\w%_!~\\-\\.\\*\\(\\);:@&=\\/\\?\\+\\$\\,#]+',
        mailPattern: '[\\w\\.\\-_]+@[\\w\\-\\._]+\\w',
        mailtoPattern: 'mailto:[\\w\\.\\-_]+@[\\w\\-\\._]+\\w(\\?[^\\r\\n]*)?', // BTMANTIS0021452 V25R11 BTMANTIS0021548
        phonePattern: '0\\d+\\-\\d+\\-\\d{4}',

        convertTextNode: function (textNode, options) {
            /// <summary>テキストノードを変換する</summary>
            /// <param name="textNode" type="Node">テキストノード</param>
            /// <param name="options" type="Object">オプション</param>
            var self = this;
            var tokens = self._splitTextToTokens(textNode.data, options);
            $.each(tokens, function (index, token) {
                self._insertWithToken(textNode, token, options);
            });
            textNode.parentNode.removeChild(textNode);
        },

        //- Private(分解) ------------------------------------------------------
        _splitTextToTokens: function (text, options) {
            /// <summary>テキストをトークン列に分解する</summary>
            /// <param name="text" type="String">文字列</param>
            /// <param name="options" type="Object">オプション</param>
            var pattern, schemas;
            var tokens = [{ type: "text", content: text}];

            if (options.convertURL) {
                schemas = options.urlSchemas.join('|');
                pattern = new RegExp('(' + schemas + ')' + this.urlPatternAfterSchema);
                tokens = this._getTokensFromTokens(tokens, "url", pattern);
            }
            if (options.convertMail) {
                // BTMANTIS0021452 V25R11 START
                pattern = new RegExp(this.mailtoPattern);
                tokens = this._getTokensFromTokens(tokens, "mailto", pattern);
                // BTMANTIS0021452 V25R11 END
                pattern = new RegExp(this.mailPattern);
                tokens = this._getTokensFromTokens(tokens, "mail", pattern);
            }
            if (options.convertPhone) {
                pattern = new RegExp(this.phonePattern);
                tokens = this._getTokensFromTokens(tokens, "phone", pattern);
            }

            return tokens;
        },

        _getTokensFromTokens: function (tokens, typeName, pattern) {
            /// <summary>トークン列を正規表現で分割したトークン列にする</summary>
            /// <param name="tokens" type="Array">トークン列</param>
            /// <param name="typeName" type="String">text|mail|url|phone</param>
            /// <param name="pattern" type="RegExp">正規表現</param>
            /// <returns type="Array" />
            var self = this, newTokens = [];
            $.each(tokens, function (i, token) {
                if (token.type == "text") {
                    var thisToken = self._getTokensFromText(token.content, typeName, pattern);
                    $.merge(newTokens, thisToken);
                } else {
                    newTokens.push(token);
                }
            });
            return newTokens;
        },

        _getTokensFromText: function (text, typeName, pattern) {
            /// <summary>文字列を正規表現で分割したトークン列にする</summary>
            /// <param name="text" type="String">分割対象文字列</param>
            /// <param name="typeName" type="String">text|mail|url|phone</param>
            /// <param name="pattern" type="RegExp">正規表現</param>
            /// <returns type="Array" />
            var tokens = [];
            var matches = text.match(pattern);

            while (!!matches && !!matches[0]) {
                var linkText = String(matches[0]);
                // 一致部分の前
                if (matches.index > 0) {
                    tokens.push({ 'type': 'text', 'content': text.substring(0, matches.index) });
                }
                // 一致部分
                tokens.push({ 'type': typeName, 'content': linkText });
                // 一致部分の後から
                text = text.substring(matches.index + linkText.length);
                matches = text.match(pattern);
            }

            // 最後のテキスト部
            if (text.length > 0) {
                tokens.push({ 'type': 'text', 'content': text });
            }
            return tokens;
        },

        //- Private(結合) ------------------------------------------------------
        _insertWithToken: function (baseNode, token, options) {
            /// <summary>トークンから生成したノードを挿入する</summary>
            /// <param name="baseNode" type="Node">挿入対象ノード</param>
            /// <param name="token" type="Object">トークン</param>
            /// <param name="options" type="Object">オプション</param>
            var node, href, text = token.content;
            if (token.type == "text") {
                // テキストノード
                node = document.createTextNode(text);
                baseNode.parentNode.insertBefore(node, baseNode);
            } else if (token.type == "url") {
                // A要素(URLメールアドレス)の追加
                node = this._insertWithValues(baseNode, text, text, options.urlFilter);
                node.target = options.urlTarget;
            } else if (token.type == "mail") {
                // A要素(メールアドレス)の追加
                href = 'mailto:' + text;
                this._insertWithValues(baseNode, text, href, options.mailFilter);
            // BTMANTIS0021452 V25R11 START
            } else if (token.type == "mailto") {
                href = text; // URLエンコードしないでそのまま設定 BTMANTIS0021552 V25R11
                this._insertWithValues(baseNode, text, href, options.mailFilter);
            // BTMANTIS0021452 V25R11 END
            } else if (token.type == "phone") {
                // A要素(電話番号)の追加
                href = "tel:" + token.content.replace(/[^0-9]/g, "");
                this._insertWithValues(baseNode, text, href, options.phoneFilter);
            }
        },

        _insertWithValues: function (baseNode, text, href, filter) {
            /// <summary>引数から生成したノードを挿入し、それを返す</summary>
            /// <param name="baseNode" type="Node">挿入対象ノード</param>
            /// <param name="text" type="String">表示内容</param>
            /// <param name="href" type="String">href属性値</param>
            /// <param name="filter" type="Function">フィルター関数</param>
            /// <returns type="Element" />
            var aElement = document.createElement("A");
            aElement.href = href;
            baseNode.parentNode.insertBefore(aElement, baseNode);
            $(aElement).text(text);
            if (typeof filter == 'function') {
                filter(aElement);
            }
            return aElement;
        }
    }

    ///
    /// プラグインメソッド
    ///
    var methods = {
        init: function (options) {
            /// <summary>初期処理</summary>
            /// <param name="options" type="Object">オプション</param>
            this.each(function () {
                _methods.convertElement(this, options);
            });
        }
    };

    ///
    /// プラグイン内部メソッド
    ///
    var _methods = {
        convertElement: function (element, options) {
            /// <summary>要素内のテキスト変換を行う</summary>
            /// <param name="element" type="Element">要素</param>
            /// <param name="options" type="Object">オプション</param>
            var nodeArr = $.merge([], element.childNodes);
            $.each(nodeArr, function (i, node) {
                if (node.nodeType == 1 && node.tagName != "A") {
                    // A以外の要素
                    _methods.convertElement(node, options);
                } else if (node.nodeType == 3 || node.nodeType == 4) {
                    // 文字ノード
                    $.detectHref.convertTextNode(node, options);
                }
            });
        }
    };

    $.fn.detectHref = function (methodNameOrOptions) {
        var isMethodName = typeof methodNameOrOptions == "string";
        // メソッド呼び出し
        if (isMethodName) {
            var methodArgs = Array.prototype.slice.call(arguments, 1);
            return methods[methodNameOrOptions].apply(this, methodArgs);
        }

        // プラグイン処理
        var customOptions = (typeof (methodNameOrOptions) == "object") ? methodNameOrOptions : {};
        var options = $.extend({}, defaultOptions, customOptions);
        return methods.init.apply(this, [options]);
    };

})(jQuery);
